/*******************************************************************************
 * Copyright 2024 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR busINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

#ifndef MODAL_IO_H
#define MODAL_IO_H

#include <stdint.h>








int modal_io_open_sink(void);
int modal_io_close_sink(void);


#define MAX_NEOPIXELS	32
int modal_io_send_neopixel_packet_generic(uint8_t* data, uint8_t n, int is_rgbw);
int modal_io_send_neopixel_packet_white(uint8_t val, int n, int is_rgbw);


int modal_io_send_esc_tone_packet(int freq_hz, int duration_ms, int pwr_percent);




// mapping of ESC aux pins to uint16_t integers
// MSB is the ESC Channel (0-3)
// LSB is split in half, left 4 bits is the pin letter (A-F -> 0-5)
// right 4 bits is the pin number
// when updating this list don't forget to change the header in config_file.c
// and the print function in modal_io.c!!
#define ESC_PIN_M0129_AUX0			((0<<8) + (0<<4) + 2)	// esc id 0, pin A2
#define ESC_PIN_M0129_AUX1			((1<<8) + (0<<4) + 2)	// esc id 1, pin A2
#define ESC_PIN_M0129_AUX2			((2<<8) + (0<<4) + 2)	// esc id 2, pin A2
#define ESC_PIN_M0129_AUX3			((3<<8) + (0<<4) + 2)	// esc id 3, pin A2
#define ESC_PIN_M0129_AUX_REG_EN	((0<<8) + (0<<4) + 6)	// esc id 0, pin A6
#define ESC_PIN_M0129_AUX_REG_ADJ	((0<<8) + (1<<4) + 6)	// esc id 0, pin B6

#define ESC_PIN_M0138_VTX_REG		((0<<8) + (5<<4) + 0)	// esc id 0, pin F0
#define ESC_PIN_M0138_RGB_OUT		((0<<8) + (1<<4) + 8)	// esc id 0, pin B8
#define ESC_PIN_M0138_AUX0			((0<<8) + (0<<4) + 2)	// esc id 0, pin A2
#define ESC_PIN_M0138_AUX1			((3<<8) + (0<<4) + 2)	// esc id 3, pin A2
#define ESC_PIN_M0138_AUX_REG_EN	((0<<8) + (0<<4) + 12)	// esc id 0, pin A12
#define ESC_PIN_M0138_AUX_REG_ADJ	((0<<8) + (0<<4) + 11)	// esc id 0, pin A11

// pins for the D0008 Drone Patrios Light bar and SUPI payload connection
#define D0008_V5_LIGHT_BAR_PIN		ESC_PIN_M0138_AUX0
#define D0008_V5_PAYLOAD_PIN_0		ESC_PIN_M0138_AUX1
#define D0008_V5_PAYLOAD_PIN_1		ESC_PIN_M0138_RGB_OUT

// don't forget to update this function when adding new pins to the above list!!!
void modal_io_print_esc_pins(void);

int modal_io_send_esc_pwm_packet(uint16_t pin, int width_us, int en_timeout);
int modal_io_send_esc_gpio_out_packet(uint16_t pin, int val);

#endif // MODAL_IO_H
